@@ -1,19 +1,18 @@
 import os
-from urlparse import urlparse, urlunparse
+from six.moves.urllib.parse import urlparse, urlunparse
 
 from w3lib.url import path_to_file_uri
-from zope.interface import implements
+from zope.interface import implementer
 
 from .interfaces import IEnvironment
 
+@implementer(IEnvironment)
 class Environment(object):
 
-    implements(IEnvironment)
-
     def __init__(self, config, initenv=os.environ):
         self.dbs_dir = config.get('dbs_dir', 'dbs')
         self.logs_dir = config.get('logs_dir', 'logs')
-        self.items_dir = config.get('items_dir','')
+        self.items_dir = config.get('items_dir', '')
         self.jobs_to_keep = config.getint('jobs_to_keep', 5)
         if config.cp.has_section('settings'):
             self.settings = dict(config.cp.items('settings'))
@@ -28,6 +27,8 @@ class Environment(object):
         env['SCRAPY_PROJECT'] = project
         env['SCRAPY_SPIDER'] = message['_spider']
         env['SCRAPY_JOB'] = message['_job']
+        if '_version' in message:
+            env['SCRAPY_EGG_VERSION'] = message['_version']
         if project in self.settings:
             env['SCRAPY_SETTINGS_MODULE'] = self.settings[project]
         if self.logs_dir:
