@@ -9,3 +9,6 @@ import scrapyd
 def main():
     argv[1:1] = ['-n', '-y', join(dirname(scrapyd.__file__), 'txapp.py')]
     run()
+
+if __name__ == '__main__':
+    main()
