@@ -1,13 +1,17 @@
+from datetime import datetime
 import sqlite3
-try:
-    import cPickle as pickle
-except:
-    import pickle
 import json
-from UserDict import DictMixin
+try:
+    from collections.abc import MutableMapping
+except ImportError:
+    from collections import MutableMapping
+import six
 
 
-class SqliteDict(DictMixin):
+from ._deprecate import deprecate_class
+
+
+class JsonSqliteDict(MutableMapping):
     """SQLite-backed dictionary"""
 
     def __init__(self, database=None, table="dict"):
@@ -39,6 +43,14 @@ class SqliteDict(DictMixin):
         self.conn.execute(q, (key,))
         self.conn.commit()
 
+    def __len__(self):
+        q = "select count(*) from %s" % self.table
+        return self.conn.execute(q).fetchone()[0]
+
+    def __iter__(self):
+        for k in self.iterkeys():
+            yield k
+
     def iterkeys(self):
         q = "select key from %s" % self.table
         return (self.decode(x[0]) for x in self.conn.execute(q))
@@ -61,32 +73,13 @@ class SqliteDict(DictMixin):
         return list(self.iteritems())
 
     def encode(self, obj):
-        return obj
+        return sqlite3.Binary(json.dumps(obj).encode('ascii'))
 
     def decode(self, obj):
-        return obj
+        return json.loads(bytes(obj).decode('ascii'))
 
 
-class PickleSqliteDict(SqliteDict):
-
-    def encode(self, obj):
-        return sqlite3.Binary(pickle.dumps(obj, protocol=2))
-
-    def decode(self, obj):
-        return pickle.loads(bytes(obj))
-
-
-class JsonSqliteDict(SqliteDict):
-
-    def encode(self, obj):
-        return sqlite3.Binary(json.dumps(obj))
-
-    def decode(self, obj):
-        return json.loads(bytes(obj))
-
-
-
-class SqlitePriorityQueue(object):
+class JsonSqlitePriorityQueue(object):
     """SQLite priority queue. It relies on SQLite concurrency support for
     providing atomic inter-process operations.
     """
@@ -149,25 +142,49 @@ class SqlitePriorityQueue(object):
         return ((self.decode(x), y) for x, y in self.conn.execute(q))
 
     def encode(self, obj):
-        return obj
+        return sqlite3.Binary(json.dumps(obj).encode('ascii'))
 
     def decode(self, text):
-        return text
+        return json.loads(bytes(text).decode('ascii'))
 
 
-class PickleSqlitePriorityQueue(SqlitePriorityQueue):
+class SqliteFinishedJobs(object):
+    """SQLite finished jobs. """
 
-    def encode(self, obj):
-        return sqlite3.Binary(pickle.dumps(obj, protocol=2))
+    def __init__(self, database=None, table="finished_jobs"):
+        self.database = database or ':memory:'
+        self.table = table
+        # about check_same_thread: http://twistedmatrix.com/trac/ticket/4040
+        self.conn = sqlite3.connect(self.database, check_same_thread=False)
+        q = "create table if not exists %s (id integer primary key, " \
+            "project text, spider text, job text, start_time datetime, end_time datetime)" % table
+        self.conn.execute(q)
 
-    def decode(self, obj):
-        return pickle.loads(bytes(obj))
+    def add(self, job):
+        args = (job.project, job.spider, job.job, job.start_time, job.end_time)
+        q = "insert into %s (project, spider, job, start_time, end_time) values (?,?,?,?,?)" % self.table
+        self.conn.execute(q, args)
+        self.conn.commit()
 
+    def clear(self, finished_to_keep=None):
+        w = ""
+        if finished_to_keep:
+            limit = len(self) - finished_to_keep
+            if limit <= 0:
+                return # nothing to delete
+            w = "where id <= (select max(id) from " \
+                "(select id from %s order by end_time limit %d))" % (self.table, limit)
+        q = "delete from %s %s" % (self.table, w)
+        self.conn.execute(q)
+        self.conn.commit()
 
-class JsonSqlitePriorityQueue(SqlitePriorityQueue):
+    def __len__(self):
+        q = "select count(*) from %s" % self.table
+        return self.conn.execute(q).fetchone()[0]
 
-    def encode(self, obj):
-        return sqlite3.Binary(json.dumps(obj))
-
-    def decode(self, obj):
-        return json.loads(bytes(obj))
+    def __iter__(self):
+        q = "select project, spider, job, start_time, end_time from %s order by end_time desc" % \
+            self.table
+        return ((j[0],j[1],j[2],
+                datetime.strptime(j[3], "%Y-%m-%d %H:%M:%S.%f"), 
+                datetime.strptime(j[4], "%Y-%m-%d %H:%M:%S.%f")) for j in self.conn.execute(q))
